﻿using System;
using System.Numerics;
using Windows.Foundation;
using Windows.Foundation.Metadata;
using Windows.Graphics.Display;
using Windows.UI.Composition;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 작성자
        /// </summary>
        private Compositor compositor = Window.Current.Compositor;

        /// <summary>
        /// 애니메이션
        /// </summary>
        private SpringVector3NaturalMotionAnimation animation;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "SpringVector3NaturalMotionAnimation 클래스 사용하기";

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 로드시 처리하기 - grid_Loaded(sender, e)

        /// <summary>
        /// 그리드 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void grid_Loaded(object sender, RoutedEventArgs e)
        {
            UpdateSpringAnimation(1.0f);
        }

        #endregion
        #region 버튼 포인터 진입시 처리하기 - button_PointerEntered(sender, e)

        /// <summary>
        /// 버튼 포인터 진입시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_PointerEntered(object sender, PointerRoutedEventArgs e)
        {
            Button button = sender as Button;

            UpdateSpringAnimation(1.5f);

            StartAnimation(button, this.animation);
        }

        #endregion
        #region 버튼 포인터 이탈시 처리하기 - button_PointerExited(sender, e)

        /// <summary>
        /// 버튼 포인터 이탈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_PointerExited(object sender, PointerRoutedEventArgs e)
        {
            Button button = sender as Button;

            UpdateSpringAnimation(1f);

            StartAnimation(button, this.animation);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 제동 비율 구하기 - GetDampingRatio()

        /// <summary>
        /// 제동 비율 구하기
        /// </summary>
        /// <returns>제동 비율</returns>
        private float GetDampingRatio()
        {
            if(this.dampingRadioButtons.SelectedItem != null)
            {
                return (float)Convert.ToDouble((this.dampingRadioButtons.SelectedItem as RadioButton).Content);
            }

            return 0.6f;
        }

        #endregion
        #region 기간 구하기 - GetPeriod()

        /// <summary>
        /// 기간 구하기
        /// </summary>
        /// <returns>기간</returns>
        private TimeSpan GetPeriod()
        {
            return TimeSpan.FromMilliseconds(this.periodSlider.Value);
        }

        #endregion
        #region 스프링 애니메이션 업데이트하기 - UpdateSpringAnimation(finalValue)

        /// <summary>
        /// 스프링 애니메이션 업데이트하기
        /// </summary>
        /// <param name="finalValue">최종 값</param>
        private void UpdateSpringAnimation(float finalValue)
        {
            if(this.animation == null)
            {
                this.animation = this.compositor.CreateSpringVector3Animation();

                this.animation.Target = "Scale";
            }

            this.animation.FinalValue   = new Vector3(finalValue);
            this.animation.DampingRatio = GetDampingRatio();
            this.animation.Period       = GetPeriod();
        }

        #endregion
        #region 애니메이션 시작하기 - StartAnimation(element, animation)

        /// <summary>
        /// 애니메이션 시작하기
        /// </summary>
        /// <param name="element">엘리먼트</param>
        /// <param name="animation">애니메이션</param>
        private void StartAnimation(UIElement element, CompositionAnimation animation)
        {
            if(ApiInformation.IsApiContractPresent("Windows.Foundation.UniversalApiContract", 7))
            {
                element.StartAnimation(animation);
            }
        }

        #endregion
    }
}